// ===============================================
// File:			    main.cpp
// Description:		OpenGL-Control for testing
// Version 1:
// Current:
// Author(s):		  Thomas Kern
// ===============================================

/*
 * Copyright (C) 1999  Thomas Kern
 */

//=====================================================================
//	INCLUDES AND DEFINES
//=====================================================================
//
#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include "resource.h"

#include "GLsettings.H"

//=====================================================================
//	GLOBALS
//=====================================================================
//
static int  winX = 0, winY = 0;
static int  winWidth = 300, winHeight = 300;

OpenGLState myOpenGLState = OpenGLState(winWidth, winHeight);
static bool readyForOpenGL = false;
static bool doubleBuffer   = false;

HDC       winDeviceContext;	              // device context
HGLRC     glRenderingContext;	            // opengl rendering context
HINSTANCE hInst;						              // aktuelle Instanz

#define   MAX_LOADSTRING 100
TCHAR     szTitle[MAX_LOADSTRING];			  // Text der Titelzeile
TCHAR     szWindowClass[MAX_LOADSTRING];	// Text der Titelzeile


//=====================================================================
//	OPENGL-FUNKTIONEN
//=====================================================================
//
// --------------------------------------------------------------------
void redraw(void)
{
  /* clear color and depth buffers */
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glLoadIdentity();

  glTranslatef(0.0, 0.0, -3.0);

  /* draw six faces of a cube */
  glBegin(GL_QUADS);
  glNormal3f( 0.0F, 0.0F, 1.0F);
  glVertex3f( 0.5F, 0.5F, 0.5F); glVertex3f(-0.5F, 0.5F, 0.5F);
  glVertex3f(-0.5F,-0.5F, 0.5F); glVertex3f( 0.5F,-0.5F, 0.5F);

  glNormal3f( 0.0F, 0.0F,-1.0F);
  glVertex3f(-0.5F,-0.5F,-0.5F); glVertex3f(-0.5F, 0.5F,-0.5F);
  glVertex3f( 0.5F, 0.5F,-0.5F); glVertex3f( 0.5F,-0.5F,-0.5F);

  glNormal3f( 0.0F, 1.0F, 0.0F);
  glVertex3f( 0.5F, 0.5F, 0.5F); glVertex3f( 0.5F, 0.5F,-0.5F);
  glVertex3f(-0.5F, 0.5F,-0.5F); glVertex3f(-0.5F, 0.5F, 0.5F);

  glNormal3f( 0.0F,-1.0F, 0.0F);
  glVertex3f(-0.5F,-0.5F,-0.5F); glVertex3f( 0.5F,-0.5F,-0.5F);
  glVertex3f( 0.5F,-0.5F, 0.5F); glVertex3f(-0.5F,-0.5F, 0.5F);

  glNormal3f( 1.0F, 0.0F, 0.0F);
  glVertex3f( 0.5F, 0.5F, 0.5F); glVertex3f( 0.5F,-0.5F, 0.5F);
  glVertex3f( 0.5F,-0.5F,-0.5F); glVertex3f( 0.5F, 0.5F,-0.5F);

  glNormal3f(-1.0F, 0.0F, 0.0F);
  glVertex3f(-0.5F,-0.5F,-0.5F); glVertex3f(-0.5F,-0.5F, 0.5F);
  glVertex3f(-0.5F, 0.5F, 0.5F); glVertex3f(-0.5F, 0.5F,-0.5F);
  glEnd();

}

//=====================================================================
//	WIN32-CALLBACKS
//=====================================================================
//
// Nachrichtenbehandlungsroutine fr "Info"-Feld.
LRESULT CALLBACK About( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam )
{
	switch( message )
	{
		case WM_INITDIALOG:
				return TRUE;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
			{
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}


//=====================================================================
//	WIN32-OPENGL-INITIALISIERUNG
//=====================================================================
//
static void GetOpenGLRC()
{
  readyForOpenGL = false;           // OpenGL-Faehigkeit wird nicht erwartet
  doubleBuffer   = false;           // Double Buffer auch nicht

  int            pixelFormatNr = 0; // gehoert zusammen
  PIXELFORMATDESCRIPTOR pixelFormat =  // gleich definieren
  {
		sizeof(PIXELFORMATDESCRIPTOR),  // Size of this structure
   	1,                              // Version of this structure
    PFD_SUPPORT_OPENGL |            // Support OpenGL calls in window
    PFD_DRAW_TO_WINDOW |            // Draw to Window (not to bitmap)
    PFD_DOUBLEBUFFER,               // Double buffered mode
    PFD_TYPE_RGBA,                  // RGBA Color mode
    16,                             // prefered color depth
    0, 0, 0, 0, 0, 0,               // color bits (ignored)
    0,                              // no alpha buffer
    0,                              // alpha bits (ignored)
    0,                              // no accumulation buffer
    0, 0, 0, 0,                     // accum bits (ignored)
    16,                             // Size of depth buffer
    0,                              // no stencil buffer
    0,                              // no auxiliary buffers
    PFD_MAIN_PLANE,                 // Draw in main plane (main layer)
    0,                              // reserved (fuer schlechte Zeiten ;-))
    0, 0, 0                         // no layer, visible, damage masks
  };              
  
  // Schau'n, ob man die gewuenschten Eigenschaften freiwillig bekommt
  pixelFormatNr = ChoosePixelFormat( winDeviceContext, &pixelFormat);

  // erster Versuch zur OpenGL Initialisierung fehlgeschlagen
  if( !pixelFormatNr ) {
    MessageBox(NULL, "ChoosePixelFormat() failed:  "
               	     "Cannot find a suitable pixel format.",
                 	   "Error", MB_OK);
  }
  // deshalb versuchen, den Wert aufzuzwingen
  else if( !SetPixelFormat(winDeviceContext, pixelFormatNr, &pixelFormat) ) {
	  MessageBox(NULL, "SetPixelFormat() failed:  "
                     "Cannot set format specified.",
                     "Error", MB_OK);
  }
  // irgendwie hat's doch geklappt
  else {
    // die Faehigkeit festhalten
    readyForOpenGL = true;
		doubleBuffer   = true;
  }
}

// --------------------------------------------------------------------
//
static void QuitOpenGL(HWND &hWnd)
{
  if(hWnd && glRenderingContext) {
    wglDeleteContext(glRenderingContext);
	  wglMakeCurrent(NULL, NULL);	  
    glRenderingContext = NULL;
    readyForOpenGL = false;
    DestroyWindow(hWnd);
  }
}


//=====================================================================
//	WIN32-GRUNDFUNKTIONEN
//=====================================================================
//
LRESULT APIENTRY WndProc( HWND hWnd, UINT message,
                          WPARAM wParam, LPARAM lParam)
{
  static int wmId, wmEvent;

  switch(message) {
    
    //========================================
    // ==> haeufige Abfragen/Aufrufe zuerst
    // Zeichnen
    case WM_PAINT:
  	  {
	      PAINTSTRUCT ps;
	      BeginPaint(hWnd, &ps);
        if(glRenderingContext) {
  		    redraw();
          SwapBuffers(winDeviceContext);
        }
	      EndPaint(hWnd, &ps);
        break;
	    }
	    break;
    
    // Groessenaenderung des Fensters
    case WM_SIZE:
  	  if(glRenderingContext) {
	      winWidth  = (int) LOWORD(lParam);
	      winHeight = (int) HIWORD(lParam);
        myOpenGLState.SetPerspective(winWidth, winHeight);
	      break;
      }

    //========================================
    // Aufrufe des Menues
		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			// Menauswahlen analysieren:
			switch( wmId ) 
			{
        // Initialisieren mit den Startwerten
        case ID_DATEI_NEUSTARTEN:
          
          break;
        // Aufruf der Info
				case ID_HILFE_INFO:
				   DialogBox(hInst, (LPCTSTR)IDD_ABOUT, hWnd, (DLGPROC)About);
				   break;
        // Programm beenden
				case ID_DATEI_BEENDEN:
				   QuitOpenGL(hWnd);
	         break;

        // keine Option ausgewaehlt
				default:
				   return DefWindowProc( hWnd, message, wParam, lParam );
			}
			break;

    //========================================
    // Aufrufe der Tastatur
    case WM_CHAR:
  	  switch( (int)wParam ) {
	      case VK_ESCAPE:
	        QuitOpenGL(hWnd);
          break;

        default:
          return DefWindowProc( hWnd, message, wParam, lParam );
      }
	    break;

    //========================================
    // ==> (einmalig) OpenGL und Windows initialisieren
    // initialize OpenGL rendering
    case WM_CREATE:      
      winDeviceContext = GetDC(hWnd); // Handle ist vom Eltern-Widget
	    GetOpenGLRC();                  // OpenGL Einstellungen setzen
      // endgueltig verbinden
      if(readyForOpenGL) {
        glRenderingContext = wglCreateContext( winDeviceContext );
        wglMakeCurrent( winDeviceContext, glRenderingContext );
      }
	    break;

    //========================================
    // finish OpenGL rendering, wird durch PostQuitMessage(0) aufgerufen
    case WM_DESTROY:      
      //if(readyForOpenGL)
      //  QuitOpenGL(hWnd);
      PostQuitMessage(0);
      break;

    //========================================
    // not defined choice
    default:
			return DefWindowProc( hWnd, message, wParam, lParam );

  } // main switch
  
  return 0; // avoid Access Violation
}


//=====================================================================
//	WIN32-MAIN
//=====================================================================
//
int APIENTRY WinMain(HINSTANCE hCurrentInst,
                     HINSTANCE hPreviousInst,
                     LPSTR     lpszCmdLine,
                     int       nCmdShow )
{
	MSG      msg;
	HACCEL   hAccelTable;
  HWND     hWnd;
  WNDCLASS wndClass;

	// Globale Zeichenfolgen initialisieren
	LoadString(hCurrentInst, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hCurrentInst, IDC_SETTINGS,  szWindowClass, MAX_LOADSTRING);

  // Initialisierung der Anwendung durchfhren:
  wndClass.style        = CS_HREDRAW | CS_VREDRAW;
  wndClass.lpfnWndProc  = (WNDPROC)WndProc;
  wndClass.cbClsExtra   = 0;
  wndClass.cbWndExtra   = 0;
  wndClass.hInstance    = hCurrentInst;
  wndClass.hIcon        = LoadIcon( hCurrentInst, MAKEINTRESOURCE(IDI_THK_ICON) );
  wndClass.hCursor      = LoadCursor(NULL, IDC_ARROW);
  wndClass.hbrBackground= (HBRUSH)GetStockObject(BLACK_BRUSH);
  wndClass.lpszMenuName = (LPCSTR)IDR_MENU1;
  wndClass.lpszClassName= szWindowClass;
  RegisterClass(&wndClass);

  // Fenster erstellen
  hWnd = CreateWindow( szWindowClass, szTitle,
	                     WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
	                     winX, winY, winWidth, winHeight,
	                     NULL, NULL, hCurrentInst, NULL);
  // Fenster anzeigen
  ShowWindow(hWnd, nCmdShow);
  UpdateWindow(hWnd);

  // Menuestruktur
  hAccelTable = LoadAccelerators(hCurrentInst, (LPCTSTR)IDR_MENU1);

	// Hauptnachrichtenschleife:
  while( GetMessage(&msg, NULL, 0, 0) ) {
    if( !TranslateAccelerator (msg.hwnd, hAccelTable, &msg) ) {
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}
	}

	return msg.wParam;
}